@@ -24,7 +24,8 @@ crew_controller_local <- function(
   tls_enable = NULL,
   tls_config = NULL,
   serialization = NULL,
-  seconds_interval = 1,
+  profile = crew::crew_random_name(),
+  seconds_interval = 0.25,
   seconds_timeout = 60,
   seconds_launch = 30,
   seconds_idle = 300,
@@ -106,6 +107,7 @@ crew_controller_local <- function(
     tls_enable = tls_enable,
     tls_config = tls_config,
     serialization = serialization,
+    profile = profile,
     seconds_interval = seconds_interval,
     seconds_timeout = seconds_timeout
   )
