@@ -11,7 +11,6 @@ crew_test("single controller, no tasks", {
       expect_true(
         x$wait(
           mode = "all",
-          seconds_interval = 5,
           seconds_timeout = Inf
         )
       )
@@ -21,7 +20,6 @@ crew_test("single controller, no tasks", {
       expect_false(
         x$wait(
           mode = "one",
-          seconds_interval = 5,
           seconds_timeout = Inf
         )
       )
@@ -48,7 +46,6 @@ crew_test("single controller, all tasks already done", {
         expect_true(
           x$wait(
             mode = mode,
-            seconds_interval = 0.5,
             seconds_timeout = Inf
           )
         )
@@ -75,7 +72,6 @@ crew_test("single controller, one long task, time out", {
         expect_false(
           x$wait(
             mode = mode,
-            seconds_interval = 0.1,
             seconds_timeout = 0.2
           )
         )
@@ -102,7 +98,6 @@ crew_test("single controller, one long task, wait all, full wait", {
         expect_true(
           x$wait(
             mode = mode,
-            seconds_interval = 0.5,
             seconds_timeout = Inf
           )
         )
