@@ -2,7 +2,7 @@ crew_test("crew_controller_group() method and signature compatibility", {
   x <- crew_controller_local()
   y <- crew_controller_group(x = x)
   common <- intersect(names(x), names(y))
-  methods <- fltr(common, ~is.function(x[[.x]]))
+  methods <- fltr(common, ~ is.function(x[[.x]]))
   methods <- setdiff(methods, "initialize")
   for (method in methods) {
     expect_equal(names(formals(x[[method]])), names(formals(y[[method]])))
@@ -13,7 +13,6 @@ crew_test("crew_controller_group() active bindings for covr", {
   x <- crew_controller_local()
   y <- crew_controller_group(x = x)
   expect_true(inherits(y$relay, "crew_class_relay"))
-  expect_true(inherits(y$throttle, "crew_class_throttle"))
 })
 
 crew_test("crew_controller_group()", {
@@ -40,27 +39,28 @@ crew_test("crew_controller_group()", {
     expect_false(x$controllers[[index]]$client$started)
   }
   expect_false(x$started())
-  expect_equal(length(x$pids()), 1L)
+  expect_equal(suppressWarnings(length(x$pids())), 1L)
   x$start()
-  expect_true(x$wait(mode = "all"))
+  expect_true(x$wait(mode = "all", seconds_timeout = 30))
+  expect_equal(x$size(), 0L)
   expect_true(x$started())
   expect_true(x$empty())
   expect_false(x$saturated())
   expect_true(x$empty(controllers = "a"))
   expect_true(x$empty(controllers = "b"))
-  expect_equal(length(x$pids()), 3L)
+  expect_equal(suppressWarnings(length(x$pids())), 1L)
   for (index in seq_len(2)) {
     expect_true(x$controllers[[index]]$client$started)
   }
   crew_retry(
-    ~{
+    ~ {
       x$wait(seconds_timeout = 30)
       TRUE
     },
     seconds_interval = 0.1,
     seconds_timeout = 5
   )
-  expect_true(x$wait(mode = "all"))
+  expect_true(x$wait(mode = "all", seconds_timeout = 30))
   s <- x$summary()
   expect_equal(nrow(s), 2L)
   expect_equal(s$controller, c("a", "b"))
@@ -69,8 +69,8 @@ crew_test("crew_controller_group()", {
     sort(
       c(
         "controller",
-        "tasks",
         "seconds",
+        "tasks",
         "success",
         "error",
         "crash",
@@ -86,6 +86,7 @@ crew_test("crew_controller_group()", {
     name = "task_pid",
     controller = "b"
   )
+  expect_equal(x$size(), 1L)
   # covr on GitHub Actions mysteriously causes problems
   # in many crew tests.
   covr_ci <- isTRUE(as.logical(Sys.getenv("R_COVR", "false"))) &&
@@ -95,7 +96,7 @@ crew_test("crew_controller_group()", {
     expect_false(x$empty())
     expect_true(x$empty(controllers = "a"))
     expect_false(x$empty(controllers = "b"))
-    x$wait(mode = "all", seconds_timeout = 5)
+    x$wait(mode = "all", seconds_timeout = 30)
     expect_false(x$empty())
     expect_true(x$empty(controllers = "a"))
     expect_false(x$empty(controllers = "b"))
@@ -113,7 +114,8 @@ crew_test("crew_controller_group()", {
     expect_true(anyNA(out$trace))
     expect_true(anyNA(out$warnings))
     pid_out <- out$result[[1]]
-    pid_exp <- x$controllers[[2]]$launcher$instances$handle[[1]]$get_pid()
+    handle <- unlist(x$controllers[[2]]$launcher$launches$handle)[[1L]]
+    pid_exp <- handle$get_pid()
     expect_equal(pid_out, pid_exp)
   }
   # substitute = FALSE # nolint
@@ -123,10 +125,11 @@ crew_test("crew_controller_group()", {
     name = "task_pid2",
     controller = "a"
   )
-  x$wait(seconds_timeout = 5)
+  x$wait(seconds_timeout = 30)
+  expect_equal(x$crashes(name = "task_pid2"), 0L)
   envir <- new.env(parent = emptyenv())
   crew_retry(
-    ~{
+    ~ {
       envir$out <- x$pop(scale = TRUE)
       !is.null(envir$out)
     },
@@ -145,17 +148,19 @@ crew_test("crew_controller_group()", {
     expect_true(anyNA(out$trace))
     expect_true(anyNA(out$warnings))
     pid_out <- out$result[[1]]
-    pid_exp <- x$controllers[[1]]$launcher$instances$handle[[1]]$get_pid()
+    pid_exp <- unlist(
+      x$controllers[[1]]$launcher$launches$handle
+    )[[1]]$get_pid()
     expect_equal(pid_out, pid_exp)
   }
   # cleanup
-  handle <- x$controllers[[2]]$launcher$instances$handle[[1]]
+  handle <- unlist(x$controllers[[2]]$launcher$launches$handle)[[1]]
   x$terminate()
   expect_false(x$started())
   for (index in seq_len(2)) {
     expect_false(x$controllers[[index]]$client$started)
     crew_retry(
-      ~!handle$is_alive(),
+      ~ !handle$is_alive(),
       seconds_interval = 0.1,
       seconds_timeout = 5
     )
@@ -176,8 +181,8 @@ crew_test("crew_controller_group() can relay task errors as local errors", {
     crew_test_sleep()
   })
   x$start()
-  x$push(command =  stop("this is an error"), name = "warnings_and_errors")
-  x$wait(seconds_timeout = 5)
+  x$push(command = stop("this is an error"), name = "warnings_and_errors")
+  x$wait(seconds_timeout = 30)
   expect_silent(
     if_any(
       isTRUE(as.logical(Sys.getenv("R_COVR", "false"))),
@@ -204,8 +209,8 @@ crew_test("crew_controller_group() can relay task errors as local warnings", {
     crew_test_sleep()
   })
   x$start()
-  x$push(command =  stop("this is an error"), name = "warnings_and_errors")
-  x$wait(seconds_timeout = 5)
+  x$push(command = stop("this is an error"), name = "warnings_and_errors")
+  x$wait(seconds_timeout = 30)
   expect_silent(
     if_any(
       isTRUE(as.logical(Sys.getenv("R_COVR", "false"))),
@@ -269,12 +274,12 @@ crew_test("crew_controller_group() launch method", {
   x$start()
   expect_silent(x$launch(n = 1L))
   handles <- list(
-    x$controllers[["a"]]$launcher$instances$handle[[1L]],
-    x$controllers[["b"]]$launcher$instances$handle[[1L]]
+    unlist(x$controllers[["a"]]$launcher$launches$handle)[[1L]],
+    unlist(x$controllers[["b"]]$launcher$launches$handle)[[1L]]
   )
   for (index in seq_len(2L)) {
     crew_retry(
-      ~handles[[index]]$is_alive(),
+      ~ handles[[index]]$is_alive(),
       seconds_interval = 0.1,
       seconds_timeout = 5
     )
@@ -282,7 +287,7 @@ crew_test("crew_controller_group() launch method", {
   x$terminate()
   for (index in seq_len(2L)) {
     crew_retry(
-      ~!handles[[index]]$is_alive(),
+      ~ !handles[[index]]$is_alive(),
       seconds_interval = 0.1,
       seconds_timeout = 5
     )
@@ -305,21 +310,21 @@ crew_test("crew_controller_group() scale method", {
   })
   x$start()
   a$push(command = "x", scale = FALSE)
-  x$scale()
+  x$scale(throttle = FALSE)
   crew_retry(
-    ~nrow(a$launcher$instances) > 0L,
+    ~ nrow(a$launcher$launches) > 0L,
     seconds_interval = 0.1,
     seconds_timeout = 5
   )
-  handle <- a$launcher$instances$handle[[1L]]
+  handle <- unlist(a$launcher$launches$handle)[[1L]]
   crew_retry(
-    fun = ~handle$is_alive(),
+    fun = ~ handle$is_alive(),
     seconds_interval = 0.1,
     seconds_timeout = 5
   )
   x$terminate()
   crew_retry(
-    fun = ~!handle$is_alive(),
+    fun = ~ !handle$is_alive(),
     seconds_interval = 0.1,
     seconds_timeout = 5
   )
@@ -352,7 +357,7 @@ crew_test("controller walk()", {
   expect_true(is.list(out))
   expect_s3_class(out[[1L]], "mirai")
   expect_s3_class(out[[2L]], "mirai")
-  x$wait(mode = "all")
+  x$wait(mode = "all", seconds_timeout = 30)
   task1 <- x$pop()
   task2 <- x$pop()
   expect_true(tibble::is_tibble(task1))
@@ -385,8 +390,10 @@ crew_test("controller group collect() with one active controller", {
   x <- crew_controller_group(a, b)
   x$push("done", controller = "a")
   x$push("done", controller = "a")
-  x$wait(mode = "all")
-  for (index in seq_len(2L)) x$push(Sys.sleep(120))
+  x$wait(mode = "all", seconds_timeout = 30)
+  for (index in seq_len(2L)) {
+    x$push(Sys.sleep(120))
+  }
   out <- x$collect()
   expect_equal(nrow(out), 2L)
   expect_equal(as.character(out$result), rep("done", 2))
@@ -415,8 +422,10 @@ crew_test("controller group collect() with two active controllers", {
   x <- crew_controller_group(a, b)
   x$push("done", controller = "a")
   x$push("done", controller = "b")
-  x$wait(mode = "all")
-  for (index in seq_len(2L)) x$push(Sys.sleep(120))
+  x$wait(mode = "all", seconds_timeout = 30)
+  for (index in seq_len(2L)) {
+    x$push(Sys.sleep(120))
+  }
   out <- x$collect()
   expect_equal(nrow(out), 2L)
   expect_equal(as.character(out$result), rep("done", 2))
@@ -438,7 +447,7 @@ crew_test("controller group collect() silent error", {
   x$push("success")
   x$push(stop("failure 1"))
   x$push(stop("failure 2"))
-  x$wait(mode = "all")
+  x$wait(mode = "all", seconds_timeout = 30)
   expect_silent(out <- x$collect(error = "silent"))
   expect_true("failure 1" %in% out$error)
 })
@@ -458,7 +467,7 @@ crew_test("controller group collect() error as warning", {
   x$push("success")
   x$push(stop("failure 1"))
   x$push(stop("failure 2"))
-  x$wait(mode = "all")
+  x$wait(mode = "all", seconds_timeout = 30)
   expect_silent(
     if_any(
       isTRUE(as.logical(Sys.getenv("R_COVR", "false"))),
@@ -485,7 +494,7 @@ crew_test("controller group collect() stop on error", {
   x$push("success")
   x$push(stop("failure 1"))
   x$push(stop("failure 2"))
-  x$wait(mode = "all")
+  x$wait(mode = "all", seconds_timeout = 30)
   expect_silent(
     if_any(
       isTRUE(as.logical(Sys.getenv("R_COVR", "false"))),
@@ -533,10 +542,7 @@ crew_test("controller group map() works", {
   expect_equal(out$error, rep(NA_character_, 2L))
   expect_equal(out$trace, rep(NA_character_, 2L))
   expect_equal(out$warnings, rep(NA_character_, 2L))
-  expect_true(is.character(out$worker))
-  expect_equal(out$controller, rep(a$launcher$name, 2L))
   sum <- x$summary()
-  expect_equal(sum$tasks, 2L)
   expect_equal(sum$success, 2L)
   expect_equal(sum$error, 0L)
   expect_equal(sum$warning, 0L)
@@ -653,8 +659,8 @@ crew_test("backlog with no tasks", {
     crew_test_sleep()
   })
   x$start()
-  expect_equal(a$backlog$list(), character(0L))
-  expect_equal(b$backlog$list(), character(0L))
+  expect_equal(as.character(a$queue_backlog$as_list()), character(0L))
+  expect_equal(as.character(b$queue_backlog$as_list()), character(0L))
   expect_equal(x$pop_backlog(), character(0L))
   tasks_a <- paste0("a_", seq_len(4L))
   for (task in tasks_a) {
@@ -664,20 +670,25 @@ crew_test("backlog with no tasks", {
   for (task in tasks_b) {
     x$push_backlog(name = task, controller = "b")
   }
-  expect_equal(a$backlog$list(), tasks_a)
-  expect_equal(b$backlog$list(), tasks_b)
-  expect_equal(
-    sort(x$pop_backlog()),
-    sort(c(tasks_a[seq_len(2L)], tasks_b[seq_len(2L)]))
-  )
-  expect_equal(a$backlog$list(), tasks_a[c(3L, 4L)])
-  expect_equal(b$backlog$list(), tasks_b[c(3L, 4L)])
-  expect_equal(
-    sort(x$pop_backlog()),
-    sort(c(tasks_a[c(3L, 4L)], tasks_b[c(3L, 4L)]))
-  )
-  expect_equal(a$backlog$list(), character(0L))
-  expect_equal(b$backlog$list(), character(0L))
+  expect_equal(as.character(a$queue_backlog$as_list()), tasks_a)
+  expect_equal(as.character(b$queue_backlog$as_list()), tasks_b)
+  out <- x$pop_backlog()
+  expect_equal(length(out), 4L)
+  expect_equal(length(unique(out)), 4L)
+  expect_equal(sum(out %in% tasks_a), 2L)
+  expect_equal(sum(out %in% tasks_b), 2L)
+  expect_equal(length(a$queue_backlog$as_list()), 2L)
+  expect_equal(length(b$queue_backlog$as_list()), 2L)
+  expect_false(any(as.character(a$queue_backlog$as_list()) %in% out))
+  expect_false(any(as.character(b$queue_backlog$as_list()) %in% out))
+  out2 <- x$pop_backlog()
+  expect_equal(length(out2), 4L)
+  expect_equal(length(unique(out2)), 4L)
+  expect_equal(sum(out2 %in% tasks_a), 2L)
+  expect_equal(sum(out2 %in% tasks_b), 2L)
+  expect_false(any(out %in% out2))
+  expect_equal(as.character(a$queue_backlog$as_list()), character(0L))
+  expect_equal(as.character(b$queue_backlog$as_list()), character(0L))
   expect_equal(x$pop_backlog(), character(0L))
 })
 
@@ -702,8 +713,8 @@ crew_test("backlog with the first controller saturated`", {
     crew_test_sleep()
   })
   x$start()
-  expect_equal(a$backlog$list(), character(0L))
-  expect_equal(b$backlog$list(), character(0L))
+  expect_equal(as.character(a$queue_backlog$as_list()), character(0L))
+  expect_equal(as.character(b$queue_backlog$as_list()), character(0L))
   expect_equal(x$pop_backlog(), character(0L))
   tasks_a <- paste0("a_", seq_len(4L))
   for (task in tasks_a) {
@@ -713,17 +724,23 @@ crew_test("backlog with the first controller saturated`", {
   for (task in tasks_b) {
     x$push_backlog(name = task, controller = "b")
   }
-  expect_equal(a$backlog$list(), tasks_a)
-  expect_equal(b$backlog$list(), tasks_b)
+  expect_equal(sort(as.character(a$queue_backlog$as_list())), sort(tasks_a))
+  expect_equal(sort(as.character(b$queue_backlog$as_list())), sort(tasks_b))
   for (index in c(1L, 2L)) {
     x$push(Sys.sleep(30), controller = "a")
   }
-  expect_equal(x$pop_backlog(), tasks_b[seq_len(2L)])
-  expect_equal(a$backlog$list(), tasks_a)
-  expect_equal(b$backlog$list(), tasks_b[c(3L, 4L)])
-  expect_equal(x$pop_backlog(), tasks_b[c(3L, 4L)])
-  expect_equal(a$backlog$list(), tasks_a)
-  expect_equal(b$backlog$list(), character(0L))
+  out <- x$pop_backlog()
+  expect_equal(length(out), 2L)
+  expect_equal(length(unique(out)), 2L)
+  expect_true(all(out %in% tasks_b))
+  expect_equal(sort(as.character(a$queue_backlog$as_list())), sort(tasks_a))
+  expect_equal(
+    sort(as.character(b$queue_backlog$as_list())),
+    sort(setdiff(tasks_b, out))
+  )
+  expect_equal(sort(x$pop_backlog()), sort(setdiff(tasks_b, out)))
+  expect_equal(sort(as.character(a$queue_backlog$as_list())), sort(tasks_a))
+  expect_equal(as.character(b$queue_backlog$as_list()), character(0L))
   expect_equal(x$pop_backlog(), character(0L))
 })
 
@@ -748,8 +765,8 @@ crew_test("backlog with the second controller saturated`", {
     crew_test_sleep()
   })
   x$start()
-  expect_equal(a$backlog$list(), character(0L))
-  expect_equal(b$backlog$list(), character(0L))
+  expect_equal(as.character(a$queue_backlog$as_list()), character(0L))
+  expect_equal(as.character(b$queue_backlog$as_list()), character(0L))
   expect_equal(x$pop_backlog(), character(0L))
   tasks_a <- paste0("a_", seq_len(4L))
   for (task in tasks_a) {
@@ -759,21 +776,28 @@ crew_test("backlog with the second controller saturated`", {
   for (task in tasks_b) {
     x$push_backlog(name = task, controller = "b")
   }
-  expect_equal(a$backlog$list(), tasks_a)
-  expect_equal(b$backlog$list(), tasks_b)
+  expect_equal(as.character(a$queue_backlog$as_list()), tasks_a)
+  expect_equal(as.character(b$queue_backlog$as_list()), tasks_b)
   for (index in c(1L, 2L)) {
     x$push(Sys.sleep(30), controller = "b")
   }
-  expect_equal(x$pop_backlog(), tasks_a[seq_len(2L)])
-  expect_equal(a$backlog$list(), tasks_a[c(3L, 4L)])
-  expect_equal(b$backlog$list(), tasks_b)
-  expect_equal(x$pop_backlog(), tasks_a[c(3L, 4L)])
-  expect_equal(a$backlog$list(), character(0L))
-  expect_equal(b$backlog$list(), tasks_b)
+  out <- x$pop_backlog()
+  expect_equal(length(out), 2L)
+  expect_equal(length(unique(out)), 2L)
+  expect_true(all(out %in% tasks_a))
+  expect_equal(
+    sort(as.character(a$queue_backlog$as_list())),
+    sort(setdiff(tasks_a, out))
+  )
+  expect_equal(sort(as.character(b$queue_backlog$as_list())), sort(tasks_b))
+  out2 <- x$pop_backlog()
+  expect_equal(sort(out2), sort(setdiff(tasks_a, out)))
+  expect_equal(as.character(a$queue_backlog$as_list()), character(0L))
+  expect_equal(sort(as.character(b$queue_backlog$as_list())), sort(tasks_b))
   expect_equal(x$pop_backlog(), character(0L))
 })
 
-crew_test("group helper methods (non)empty, (un)resolved, unpopped", {
+crew_test("group helper methods (non)empty and (un)resolved", {
   skip_on_cran()
   skip_on_os("windows")
   a <- crew_controller_local(
@@ -798,35 +822,31 @@ crew_test("group helper methods (non)empty, (un)resolved, unpopped", {
   expect_false(x$nonempty())
   expect_equal(x$resolved(), 0L)
   expect_equal(x$unresolved(), 0L)
-  expect_equal(x$unpopped(), 0L)
   x$push(TRUE, controller = "a")
   x$push(TRUE, controller = "b")
-  x$wait(mode = "all")
+  x$wait(mode = "all", seconds_timeout = 30)
   expect_false(x$empty())
   expect_true(x$nonempty())
   expect_equal(x$resolved(), 2L)
   expect_equal(x$unresolved(), 0L)
-  expect_equal(x$unpopped(), 2L)
   tasks <- x$collect()
   expect_true(x$empty())
   expect_false(x$nonempty())
-  expect_equal(x$unpopped(), 0L)
   x$push(Sys.sleep(60), controller = "a")
   x$push(Sys.sleep(60), controller = "b")
   expect_false(x$empty())
   expect_true(x$nonempty())
   expect_equal(x$resolved(), 2L)
   expect_equal(x$unresolved(), 2L)
-  expect_equal(x$unpopped(), 0L)
   x$terminate()
 })
 
 crew_test("descale", {
   controller <- crew_controller_local()
   x <- crew_controller_group(controller)
-  expect_false(controller$autoscaling)
+  expect_null(controller$loop)
   x$descale()
-  expect_false(controller$autoscaling)
+  expect_null(controller$loop)
 })
 
 crew_test("crash detection with backup controllers in a group", {
@@ -871,18 +891,19 @@ crew_test("crash detection with backup controllers in a group", {
     crew_retry(
       ~ {
         x$scale()
-        isTRUE(a$launcher$instances$online) ||
-          isTRUE(b$launcher$instances$online) ||
-          isTRUE(c$launcher$instances$online)
+        isTRUE(a$client$status()["connections"] > 0L) ||
+          isTRUE(b$client$status()["connections"] > 0L) ||
+          isTRUE(c$client$status()["connections"] > 0L)
       },
       seconds_interval = 0.1,
       seconds_timeout = 60
     )
-    Sys.sleep(0.25)
-    a$launcher$terminate_workers()
-    b$launcher$terminate_workers()
-    c$launcher$terminate_workers()
-    x$wait()
+    for (controller in list(a, b, c)) {
+      for (handle in unlist(controller$launcher$launches$handle)) {
+        handle$kill()
+      }
+    }
+    x$wait(mode = "one", seconds_timeout = 30, scale = FALSE)
     x$pop()
   }
   out <- crash()
@@ -894,7 +915,6 @@ crew_test("crash detection with backup controllers in a group", {
   expect_equal(c$crashes(name = "x"), 0L)
   summary <- x$summary()
   expect_equal(summary$controller, c("a", "b", "c"))
-  expect_equal(summary$tasks, c(1L, 0L, 0L))
   expect_equal(summary$crash, c(1L, 0L, 0L))
   expect_equal(summary$error, c(0L, 0L, 0L))
   out <- crash()
@@ -906,7 +926,6 @@ crew_test("crash detection with backup controllers in a group", {
   expect_equal(c$crashes(name = "x"), 0L)
   summary <- x$summary()
   expect_equal(summary$controller, c("a", "b", "c"))
-  expect_equal(summary$tasks, c(2L, 0L, 0L))
   expect_equal(summary$crash, c(2L, 0L, 0L))
   expect_equal(summary$error, c(0L, 0L, 0L))
   out <- crash()
@@ -918,7 +937,6 @@ crew_test("crash detection with backup controllers in a group", {
   expect_equal(c$crashes(name = "x"), 0L)
   summary <- x$summary()
   expect_equal(summary$controller, c("a", "b", "c"))
-  expect_equal(summary$tasks, c(2L, 1L, 0L))
   expect_equal(summary$crash, c(2L, 1L, 0L))
   out <- crash()
   expect_true(tibble::is_tibble(out))
@@ -929,7 +947,6 @@ crew_test("crash detection with backup controllers in a group", {
   expect_equal(c$crashes(name = "x"), 0L)
   summary <- x$summary()
   expect_equal(summary$controller, c("a", "b", "c"))
-  expect_equal(summary$tasks, c(2L, 2L, 0L))
   expect_equal(summary$crash, c(2L, 2L, 0L))
   out <- crash()
   expect_true(tibble::is_tibble(out))
@@ -940,7 +957,6 @@ crew_test("crash detection with backup controllers in a group", {
   expect_equal(c$crashes(name = "x"), 1L)
   summary <- x$summary()
   expect_equal(summary$controller, c("a", "b", "c"))
-  expect_equal(summary$tasks, c(2L, 2L, 1L))
   expect_equal(summary$crash, c(2L, 2L, 1L))
   out <- crash()
   expect_true(tibble::is_tibble(out))
@@ -951,7 +967,6 @@ crew_test("crash detection with backup controllers in a group", {
   expect_equal(c$crashes(name = "x"), 2L)
   summary <- x$summary()
   expect_equal(summary$controller, c("a", "b", "c"))
-  expect_equal(summary$tasks, c(2L, 2L, 2L))
   expect_equal(summary$crash, c(2L, 2L, 2L))
   expect_crew_error(crash())
 })
