@@ -1,9 +1,5 @@
 crew_test("fltr()", {
-  expect_equal(fltr(seq_len(10), ~.x < 5), seq_len(4))
-})
-
-crew_test("map()", {
-  expect_equal(unname(map(letters, identity)), as.list(letters))
+  expect_equal(fltr(seq_len(10), ~ .x < 5), seq_len(4))
 })
 
 crew_test("map_chr()", {
@@ -25,11 +21,5 @@ crew_test("map_lgl()", {
 
 crew_test("map_rows()", {
   x <- data.frame(x = seq_len(3), y = rep(1, 3), z = rep(2, 3))
-  expect_equal(map_rows(x, ~sum(.x)), seq_len(3) + 3)
-})
-
-crew_test("walk", {
-  envir <- new.env(parent = emptyenv())
-  walk(letters, ~assign(.x, 1, envir = envir))
-  expect_equal(sort(names(envir)), sort(letters))
+  expect_equal(map_rows(x, ~ sum(.x)), seq_len(3) + 3)
 })
